@@ -1,9 +1,9 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 
 import gtk
 import gobject
+import glib
 import pango
 import gettext
 import os
@@ -37,6 +37,13 @@ except ImportError:
     ssl = None
 import dis
 from threading import Lock, Semaphore
+try:
+    import pytz
+except ImportError:
+    pytz = None
+
+from tryton.exceptions import (TrytonServerError, TrytonError,
+    TrytonServerUnavailable)
 
 _ = gettext.gettext
 
@@ -59,11 +66,15 @@ class TrytonIconFactory(gtk.IconFactory):
             try:
                 pixbuf = gtk.gdk.pixbuf_new_from_file(
                         os.path.join(PIXMAPS_DIR, fname).decode('utf-8'))
-            except Exception:
+            except IOError:
                 continue
             icon_set = gtk.IconSet(pixbuf)
             self.add(name, icon_set)
             self._locale_icons.add(name)
+        for name in ('ok', 'cancel'):
+            icon_set = gtk.Style().lookup_icon_set('gtk-%s' % name)
+            self.add('tryton-%s' % name, icon_set)
+            self._locale_icons.add('tryton-%s' % name)
 
     def load_icons(self, refresh=False):
         if not refresh:
@@ -74,7 +85,7 @@ class TrytonIconFactory(gtk.IconFactory):
         try:
             icons = rpc.execute('model', 'ir.ui.icon', 'list_icons',
                 rpc.CONTEXT)
-        except Exception:
+        except TrytonServerError:
             icons = []
         for icon_id, icon_name in icons:
             if refresh and icon_name in self._loaded_icons:
@@ -91,12 +102,13 @@ class TrytonIconFactory(gtk.IconFactory):
             self.load_icons(refresh=True)
         icon_ref = (self._name2id[iconname], iconname)
         idx = self._tryton_icons.index(icon_ref)
-        to_load = slice(max(0, idx-self.batchnum/2), idx+self.batchnum/2)
+        to_load = slice(max(0, idx - self.batchnum // 2),
+            idx + self.batchnum // 2)
         ids = [e[0] for e in self._tryton_icons[to_load]]
         try:
             icons = rpc.execute('model', 'ir.ui.icon', 'read', ids,
                 ['name', 'icon'], rpc.CONTEXT)
-        except Exception:
+        except TrytonServerError:
             icons = []
         for icon in icons:
             # svg file cannot be loaded from data into a pixbuf
@@ -148,7 +160,10 @@ def refresh_dblist(host, port):
 def refresh_langlist(lang_widget, host, port):
     liststore = lang_widget.get_model()
     liststore.clear()
-    lang_list = rpc.db_exec(host, port, 'list_lang')
+    try:
+        lang_list = rpc.db_exec(host, port, 'list_lang')
+    except socket.error:
+        return []
     from tryton.gui.main import Main
     Main.get_main().refresh_ssl()
     index = -1
@@ -164,8 +179,9 @@ def refresh_langlist(lang_widget, host, port):
     lang_widget.set_active(index)
     return lang_list
 
-def request_server(server_widget, parent):
+def request_server(server_widget):
     result = False
+    parent = get_toplevel_window()
     dialog = gtk.Dialog(
         title= _('Tryton Connection'),
         parent=parent,
@@ -185,7 +201,7 @@ def request_server(server_widget, parent):
         xoptions=gtk.FILL)
     entry_port = gtk.Entry()
     entry_port.set_max_length(5)
-    entry_port.set_text("8070")
+    entry_port.set_text("8000")
     entry_port.set_activates_default(True)
     entry_port.set_width_chars(16)
     table.attach(entry_port, 1, 2, 1, 2, yoptions=False,
@@ -225,14 +241,42 @@ def request_server(server_widget, parent):
     dialog.destroy()
     return result
 
+def get_toplevel_window():
+    windows = [x for x in gtk.window_list_toplevels()
+        if x.window and x.props.visible]
+    trans2windows = dict((x.get_transient_for(), x) for x in windows)
+    for window in set(windows) - set(trans2windows.iterkeys()):
+        return window
+    return trans2windows[None]
 
-def selection(title, values, parent, alwaysask=False):
+def get_sensible_widget(window):
+    from tryton.gui.main import Main
+    main = Main.get_main()
+    if main and window == main.window:
+        focus_widget = window.get_focus()
+        page = main.get_page()
+        if page and focus_widget and focus_widget.is_ancestor(page.widget):
+            return page.widget
+    return window
+
+def center_window(window, parent, sensible):
+    parent_x, parent_y = parent.window.get_origin()
+    window_allocation = window.get_allocation()
+    sensible_allocation = sensible.get_allocation()
+    x = (parent_x + sensible_allocation.x +
+        int((sensible_allocation.width - window_allocation.width) / 2))
+    y = (parent_y + sensible_allocation.y +
+        int((sensible_allocation.height - window_allocation.height) / 2))
+    window.move(x, y)
+
+def selection(title, values, alwaysask=False):
     if not values or len(values)==0:
         return None
     elif len(values)==1 and (not alwaysask):
         key = values.keys()[0]
         return (key, values[key])
 
+    parent = get_toplevel_window()
     dialog = gtk.Dialog(_('Selection'), parent,
             gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
             (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
@@ -286,9 +330,10 @@ def selection(title, values, parent, alwaysask=False):
     dialog.destroy()
     return res
 
-def file_selection(title, filename='', parent=None,
+def file_selection(title, filename='',
         action=gtk.FILE_CHOOSER_ACTION_OPEN, preview=True, multi=False,
         filters=None):
+    parent = get_toplevel_window()
     if action == gtk.FILE_CHOOSER_ACTION_OPEN:
         buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
             gtk.STOCK_OPEN,gtk.RESPONSE_OK)
@@ -308,14 +353,16 @@ def file_selection(title, filename='', parent=None,
             win.add_filter(filt)
 
     def update_preview_cb(win, img):
+        have_preview = False
         filename = win.get_preview_filename()
-        try:
-            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(filename.decode(
-                sys.getfilesystemencoding().encode('utf-8')), 128, 128)
-            img.set_from_pixbuf(pixbuf)
-            have_preview = True
-        except Exception:
-            have_preview = False
+        if filename:
+            try:
+                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(filename.decode(
+                        sys.getfilesystemencoding().encode('utf-8')), 128, 128)
+                img.set_from_pixbuf(pixbuf)
+                have_preview = True
+            except (IOError, glib.GError):
+                pass
         win.set_preview_widget_active(have_preview)
         return
 
@@ -337,7 +384,7 @@ def file_selection(title, filename='', parent=None,
                 CONFIG['client.default_path'] = \
                         os.path.dirname(filepath)
                 CONFIG.save()
-            except Exception:
+            except IOError:
                 pass
         parent.present()
         win.destroy()
@@ -349,42 +396,37 @@ def file_selection(title, filename='', parent=None,
             try:
                 CONFIG['client.default_path'] = \
                         os.path.dirname(filenames[0])
-            except Exception:
+            except IOError:
                 pass
         parent.present()
         win.destroy()
         return filenames
 
-def file_open(filename, type, parent, print_p=False):
+def file_open(filename, type, print_p=False):
     if os.name == 'nt':
         operation = 'open'
         if print_p:
             operation = 'print'
         try:
             os.startfile(os.path.normpath(filename), operation)
-        except Exception:
-            # Try without operation, it is not supported on version < 2.5
-            try:
-                os.startfile(os.path.normpath(filename))
-            except Exception:
-                save_name = file_selection(_('Save As...'), parent=parent,
-                        action=gtk.FILE_CHOOSER_ACTION_SAVE)
-                if save_name:
-                    file_p = open(filename, 'rb')
-                    save_p = open(save_name, 'wb+')
-                    save_p.write(file_p.read())
-                    save_p.close()
-                    file_p.close()
+        except WindowsError:
+            save_name = file_selection(_('Save As...'),
+                    action=gtk.FILE_CHOOSER_ACTION_SAVE)
+            if save_name:
+                file_p = open(filename, 'rb')
+                save_p = open(save_name, 'wb+')
+                save_p.write(file_p.read())
+                save_p.close()
+                file_p.close()
         return
-    elif os.name == 'mac' or \
-            (hasattr(os, 'uname') and os.uname()[0] == 'Darwin'):
+    elif sys.platform == 'darwin':
         pid = os.fork()
         if not pid:
             pid = os.fork()
             if not pid:
                 try:
                     os.execv('/usr/bin/open', ['/usr/bin/open', filename])
-                except Exception:
+                except OSError:
                     sys.exit(0)
             time.sleep(0.1)
             sys.exit(0)
@@ -402,7 +444,7 @@ def file_open(filename, type, parent, print_p=False):
         #TODO add dialog box
         pass
     if not cmd:
-        save_name = file_selection(_('Save As...'), parent=parent,
+        save_name = file_selection(_('Save As...'),
                 action=gtk.FILE_CHOOSER_ACTION_SAVE)
         if save_name:
             file_p = open(filename, 'rb')
@@ -424,7 +466,7 @@ def file_open(filename, type, parent, print_p=False):
         if not pid:
             try:
                 os.execv(prog, args)
-            except Exception:
+            except OSError:
                 sys.exit(0)
         time.sleep(0.1)
         sys.exit(0)
@@ -451,7 +493,7 @@ def mailto(to=None, cc=None, subject=None, body=None, attachment=None):
             if not pid:
                 try:
                     os.execv(prog, args)
-                except Exception:
+                except OSError:
                     sys.exit(0)
             time.sleep(0.1)
             sys.exit(0)
@@ -496,8 +538,8 @@ class UniqueDialog(object):
         if self.running:
             return
 
-        parent = args[1]
-        dialog = self.build_dialog(*args)
+        parent = get_toplevel_window()
+        dialog = self.build_dialog(parent, *args)
         dialog.set_icon(TRYTON_ICON)
         self.running = True
         dialog.show_all()
@@ -510,21 +552,21 @@ class UniqueDialog(object):
 
 class MessageDialog(UniqueDialog):
 
-    def build_dialog(self, message, parent, msg_type):
+    def build_dialog(self, parent, message, msg_type):
         dialog = gtk.MessageDialog(parent,
             gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT, msg_type,
             gtk.BUTTONS_OK, message)
         return dialog
 
-    def __call__(self, message, parent, msg_type=gtk.MESSAGE_INFO):
-        super(MessageDialog, self).__call__(message, parent, msg_type)
+    def __call__(self, message, msg_type=gtk.MESSAGE_INFO):
+        super(MessageDialog, self).__call__(message, msg_type)
 
 message = MessageDialog()
 
 
 class WarningDialog(UniqueDialog):
 
-    def build_dialog(self, message, parent, title, buttons=gtk.BUTTONS_OK):
+    def build_dialog(self, parent, message, title, buttons=gtk.BUTTONS_OK):
         dialog = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
             gtk.MESSAGE_WARNING, buttons)
         if hasattr(dialog, 'format_secondary_markup'):
@@ -546,8 +588,8 @@ class UserWarningDialog(WarningDialog):
     def _set_always(self, toggle):
         self.always = toggle.get_active()
 
-    def build_dialog(self, message, parent, title):
-        dialog = super(UserWarningDialog, self).build_dialog(message, parent,
+    def build_dialog(self, parent, message, title):
+        dialog = super(UserWarningDialog, self).build_dialog(parent, message,
             title, gtk.BUTTONS_OK_CANCEL)
         check = gtk.CheckButton(_('Always ignore this warning.'))
         check.connect_after('toggled', self._set_always)
@@ -556,9 +598,8 @@ class UserWarningDialog(WarningDialog):
         dialog.vbox.pack_end(alignment, True, False)
         return dialog
 
-    def __call__(self, message, parent, title):
-        response = super(UserWarningDialog, self).__call__(message, parent,
-            title)
+    def __call__(self, message, title):
+        response = super(UserWarningDialog, self).__call__(message, title)
         if response == gtk.RESPONSE_OK:
             if self.always:
                 return 'always'
@@ -570,7 +611,7 @@ userwarning = UserWarningDialog()
 
 class ConfirmationDialog(UniqueDialog):
 
-    def build_dialog(self, message, parent):
+    def build_dialog(self, parent, message):
         dialog = gtk.Dialog(_('Confirmation'), parent, gtk.DIALOG_MODAL
                 | gtk.DIALOG_DESTROY_WITH_PARENT | gtk.WIN_POS_CENTER_ON_PARENT
                 | gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
@@ -584,22 +625,21 @@ class ConfirmationDialog(UniqueDialog):
         label = gtk.Label('%s' % (to_xml(message)))
         hbox.pack_start(label, True, True)
         dialog.vbox.pack_start(hbox)
-        dialog.set_transient_for(parent)
         return dialog
 
 
 class SurDialog(ConfirmationDialog):
 
-    def build_dialog(self, message, parent):
-        dialog = super(SurDialog, self).build_dialog(message, parent)
+    def build_dialog(self, parent, message):
+        dialog = super(SurDialog, self).build_dialog(parent, message)
         dialog.add_button("gtk-cancel", gtk.RESPONSE_CANCEL)
         dialog.add_button("gtk-ok", gtk.RESPONSE_OK | gtk.CAN_DEFAULT
                 | gtk.HAS_DEFAULT)
         dialog.set_default_response(gtk.RESPONSE_OK)
         return dialog
 
-    def __call__(self, message, parent):
-        response = super(SurDialog, self).__call__(message, parent)
+    def __call__(self, message):
+        response = super(SurDialog, self).__call__(message)
         return response == gtk.RESPONSE_OK
 
 sur = SurDialog()
@@ -613,8 +653,8 @@ class Sur3BDialog(ConfirmationDialog):
         gtk.RESPONSE_CANCEL: 'cancel'
     }
 
-    def build_dialog(self, message, parent):
-        dialog = super(Sur3BDialog, self).build_dialog(message, parent)
+    def build_dialog(self, parent, message):
+        dialog = super(Sur3BDialog, self).build_dialog(parent, message)
         dialog.add_button("gtk-cancel", gtk.RESPONSE_CANCEL)
         dialog.add_button("gtk-no", gtk.RESPONSE_NO)
         dialog.add_button("gtk-yes", gtk.RESPONSE_YES | gtk.CAN_DEFAULT
@@ -622,8 +662,8 @@ class Sur3BDialog(ConfirmationDialog):
         dialog.set_default_response(gtk.RESPONSE_YES)
         return dialog
 
-    def __call__(self, message, parent):
-        response = super(Sur3BDialog, self).__call__(message, parent)
+    def __call__(self, message):
+        response = super(Sur3BDialog, self).__call__(message)
         return self.response_mapping.get(response, 'cancel')
 
 sur_3b = Sur3BDialog()
@@ -631,7 +671,7 @@ sur_3b = Sur3BDialog()
 
 class AskDialog(UniqueDialog):
 
-    def build_dialog(self, question, parent, visibility):
+    def build_dialog(self, parent, question, visibility):
         win = gtk.Dialog('Tryton', parent,
                 gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                 (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
@@ -654,12 +694,12 @@ class AskDialog(UniqueDialog):
         win.vbox.pack_start(hbox)
         return win
 
-    def __call__(self, question, parent, visibility=True):
+    def __call__(self, question, visibility=True):
         if self.running:
             return
 
-        dialog = self.build_dialog(question, parent,
-            visibility=visibility)
+        parent = get_toplevel_window()
+        dialog = self.build_dialog(parent, question, visibility=visibility)
         dialog.set_icon(TRYTON_ICON)
         self.running = True
         dialog.show_all()
@@ -677,7 +717,7 @@ ask = AskDialog()
 
 class ConcurrencyDialog(UniqueDialog):
 
-    def build_dialog(self, resource, parent, obj_id, context):
+    def build_dialog(self, parent, resource, obj_id, context):
         dialog = gtk.Dialog(_('Concurrency Exception'), parent, gtk.DIALOG_MODAL
                 | gtk.DIALOG_DESTROY_WITH_PARENT | gtk.WIN_POS_CENTER_ON_PARENT
                 | gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
@@ -713,9 +753,8 @@ class ConcurrencyDialog(UniqueDialog):
         dialog.add_action_widget(write_button, gtk.RESPONSE_OK)
         return dialog
 
-    def __call__(self, resource, obj_id, context, parent):
-        # parent must be the second parameter
-        res = super(ConcurrencyDialog, self).__call__(resource, parent, obj_id,
+    def __call__(self, resource, obj_id, context):
+        res = super(ConcurrencyDialog, self).__call__(resource, obj_id,
             context)
 
         if res == gtk.RESPONSE_OK:
@@ -723,7 +762,7 @@ class ConcurrencyDialog(UniqueDialog):
         if res == gtk.RESPONSE_APPLY:
             from tryton.gui.window import Window
             Window.create(False, resource, res_id=obj_id,
-                domain=[('id', '=', obj_id)], window=parent,
+                domain=[('id', '=', obj_id)],
                 context=context, mode=['form', 'tree'])
         return False
 
@@ -732,7 +771,7 @@ concurrency = ConcurrencyDialog()
 
 class ErrorDialog(UniqueDialog):
 
-    def build_dialog(self, title, parent, details):
+    def build_dialog(self, parent, title, details):
         dialog = gtk.Dialog(_('Error'), parent,
             gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
         dialog.set_has_separator(True)
@@ -801,20 +840,21 @@ class ErrorDialog(UniqueDialog):
         dialog.set_size_request(600, 400)
         return dialog
 
-    def __call__(self, title, parent, details):
+    def __call__(self, title, details):
         if title == details:
             title = ''
         log = logging.getLogger('common.message')
         log.error(details)
 
-        response = super(ErrorDialog, self).__call__(title, parent, details)
+        response = super(ErrorDialog, self).__call__(title, details)
         if response == gtk.RESPONSE_OK:
-            send_bugtracker(details, parent)
+            send_bugtracker(details)
 
 error = ErrorDialog()
 
-def send_bugtracker(msg, parent):
+def send_bugtracker(msg):
     from tryton import rpc
+    parent = get_toplevel_window()
     win = gtk.Dialog(_('Bug Tracker'), parent,
             gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
             (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
@@ -897,7 +937,7 @@ def send_bugtracker(msg, parent):
                 server.set('issue' + str(issue_id), *['nosy=+' + user])
                 message(_('The same bug was already reported by another user.\n' \
                         'To keep you informed your username is added to the nosy-list of this issue') + \
-                        '%s' % issue_id, parent)
+                        '%s' % issue_id)
             else:
                 # create a new issue for this error-message
                 # first create message
@@ -907,120 +947,120 @@ def send_bugtracker(msg, parent):
                 issue_id = server.create('issue', *['messages=' + str(msg_id),
                     'nosy=' + user, 'title=' + title, 'priority=bug'])
                 message(_('Created new bug with ID ') + \
-                        'issue%s' % issue_id, parent)
+                        'issue%s' % issue_id)
             webbrowser.open(CONFIG['roundup.url'] + 'issue%s' % issue_id, new=2)
-        except Exception, exception:
-            if hasattr(exception, 'faultString') \
-                    and 'roundup.cgi.exceptions.Unauthorised' in exception.faultString:
-                message(_('Connection error!\n' \
-                        'Bad username or password!'), parent)
+        except (socket.error, xmlrpclib.Fault), exception:
+            if (isinstance(exception, xmlrpclib.Fault)
+                    and 'roundup.cgi.exceptions.Unauthorised' in
+                    exception.faultString):
+                message(_('Connection error!\nBad username or password!'))
                 return send_bugtracker(msg, parent)
             tb_s = reduce(lambda x, y: x + y,
                     traceback.format_exception(sys.exc_type,
                         sys.exc_value, sys.exc_traceback))
-            message(_('Exception:') + '\n' + tb_s, parent,
-                    msg_type=gtk.MESSAGE_ERROR)
+            message(_('Exception:') + '\n' + tb_s, msg_type=gtk.MESSAGE_ERROR)
 
 def to_xml(string):
     return string.replace('&','&amp;').replace('<','&lt;').replace('>','&gt;')
 
 PLOCK = Lock()
 
-def process_exception(exception, parent, *args):
-    global _USERNAME, _DATABASE, _SOCK
-    if str(exception.args[0]) == 'BadFingerprint':
-        warning(_('The server fingerprint has changed since last connection!\n'
-            'The application will stop connecting to this server '
-            'until its fingerprint is fixed.'), parent, _('Security risk!'))
-        from tryton.gui.main import Main
-        Main.sig_quit()
-        sys.exit()
-    if str(exception.args[0]) == 'NotLogged':
-        if not rpc._SOCK:
-            message(_('Connection error!\n' \
-                    'Unable to connect to the server!'), parent)
-            return False
-        if not PLOCK.acquire(False):
-            return False
-        hostname = rpc._SOCK.hostname
-        port = rpc._SOCK.port
-        try:
-            while True:
-                password = ask(_('Password:'), parent, visibility=False)
-                if password is None:
-                    raise Exception('NotLogged')
-                res = rpc.login(rpc._USERNAME, password, hostname, port,
-                        rpc._DATABASE)
-                from tryton.gui.main import Main
-                Main.get_main().refresh_ssl()
-                if res == -1:
-                    message(_('Connection error!\n' \
-                            'Unable to connect to the server!'), parent)
-                    return False
-                if res < 0:
-                    continue
+def process_exception(exception, *args, **kwargs):
+
+    if isinstance(exception, TrytonError):
+        if exception.faultCode == 'BadFingerprint':
+            warning(
+                _('The server fingerprint has changed since last connection!\n'
+                'The application will stop connecting to this server '
+                'until its fingerprint is fixed.'), _('Security risk!'))
+            from tryton.gui.main import Main
+            Main.sig_quit()
+            sys.exit()
+        elif exception.faultCode == 'NotLogged':
+            if rpc.CONNECTION is None:
+                message(_('Connection error!\n' \
+                        'Unable to connect to the server!'))
+                return False
+    elif isinstance(exception, TrytonServerError):
+        if exception.faultCode == 'UserWarning':
+            name, msg, description = exception.args
+            res = userwarning(description, msg)
+            if res in ('always', 'ok'):
+                args2 = ('model', 'res.user.warning', 'create', {
+                        'user': rpc._USER,
+                        'name': name,
+                        'always': (res == 'always'),
+                        }, rpc.CONTEXT)
+                try:
+                    rpc.execute(*args2)
+                except TrytonServerError, exception:
+                    process_exception(exception, *args2)
                 if args:
                     try:
                         return rpc.execute(*args)
-                    except Exception, exception:
-                        return process_exception(exception, parent, *args)
+                    except TrytonServerError, exception:
+                        return process_exception(exception, *args)
                 return True
-        finally:
-            PLOCK.release()
-
-    if exception.args[0] == 'ConcurrencyException':
-        if len(args) >= 6:
-            if concurrency(args[1], args[3][0], args[5], parent):
-                if '_timestamp' in args[5]:
-                    del args[5]['_timestamp']
-                try:
-                    return rpc.execute(*args)
-                except Exception, exception:
-                    return process_exception(exception, parent, *args)
             return False
-        else:
-            message(_('Concurrency Exception'), parent,
-                    msg_type=gtk.MESSAGE_ERROR)
+        elif exception.faultCode == 'UserError':
+            msg, description = exception.args
+            warning(description, msg)
             return False
-
-    if exception.args[0] == 'UserWarning':
-        msg = ''
-        if len(exception.args) > 4:
-            msg = exception.args[3]
-        res = userwarning(str(msg), parent, str(exception.args[2]))
-        if res in ('always', 'ok'):
-            args2 = ('model', 'res.user.warning', 'create', {
-                    'user': rpc._USER,
-                    'name': exception.args[1],
-                    'always': (res == 'always'),
-                    }, rpc.CONTEXT)
+        elif exception.faultCode == 'ConcurrencyException':
+            if len(args) >= 6:
+                if concurrency(args[1], args[3][0], args[5]):
+                    if '_timestamp' in args[5]:
+                        del args[5]['_timestamp']
+                    try:
+                        return rpc.execute(*args)
+                    except TrytonServerError, exception:
+                        return process_exception(exception, *args)
+                return False
+            else:
+                message(_('Concurrency Exception'), msg_type=gtk.MESSAGE_ERROR)
+                return False
+        elif exception.faultCode == 'NotLogged':
+            from tryton.gui.main import Main
+            if not PLOCK.acquire(False):
+                return False
+            hostname = rpc._HOST
+            port = rpc._PORT
             try:
-                rpc.execute(*args2)
-            except Exception, exception:
-                process_exception(exception, parent, *args2)
-            if args:
-                try:
-                    return rpc.execute(*args)
-                except Exception, exception:
-                    return process_exception(exception, parent, *args)
-            return True
+                while True:
+                    password = ask(_('Password:'), visibility=False)
+                    if password is None:
+                        Main.get_main().sig_logout()
+                        raise TrytonError('NotLogged')
+                    res = rpc.login(rpc._USERNAME, password, hostname, port,
+                            rpc._DATABASE)
+                    Main.get_main().refresh_ssl()
+                    if res == -1:
+                        message(_('Connection error!\n' \
+                                'Unable to connect to the server!'))
+                        return False
+                    if res < 0:
+                        continue
+                    if args:
+                        try:
+                            return rpc.execute(*args)
+                        except TrytonServerError, exception:
+                            return process_exception(exception, *args)
+                    return True
+            finally:
+                PLOCK.release()
+    elif isinstance(exception, (socket.error, TrytonServerUnavailable)):
+        warning(str(exception), _('Network Error!'))
         return False
 
-    if exception.args[0] == 'UserError':
-        msg = ''
-        if len(exception.args) > 3:
-            msg = exception.args[2]
-        warning(str(msg), parent, str(exception.args[1]))
-        return False
-
-    if isinstance(exception, socket.error):
-        msg = ''
-        if len(exception.args) > 2:
-            msg = exception.args[1]
-        warning(msg, parent, _('Network Error!'))
-        return False
-
-    error(str(exception.args[0]), parent, str(exception.args[-1]))
+    if isinstance(exception, TrytonServerError):
+        error_title, error_detail = exception.faultCode, exception.faultString
+    else:
+        error_title = str(exception)
+        if 'tb' in kwargs:
+            error_detail = kwargs['tb']
+        else:
+            error_detail = traceback.format_exc()
+    error(error_title, error_detail)
     return False
 
 def node_attributes(node):
@@ -1083,59 +1123,59 @@ class DBProgress(object):
         self.db_info = None
 
     def start(self):
-        key = (self.host, self.port)
-        dbs, createdb = [], False
+        dbs, createdb = None, False
         try:
             dbs = refresh_dblist(self.host, self.port)
             createdb = True
+        except Exception:
+            pass
         finally:
             self.db_info = (dbs, createdb)
             self.updated.set()
 
-    def update(self, combo, progressbar, dbname=''):
-        key = (self.host, self.port)
+    def update(self, combo, progressbar, callback, dbname=''):
         self.db_info = None
         thread = threading.Thread(target=self.start).start()
+        gobject.timeout_add(100, self.end, combo, progressbar, callback,
+            dbname)
 
-        i = 0
-        while not self.updated.isSet():
-            i += 1
-            time.sleep(0.1)
-            if i >= 10:
-                progressbar.show()
-                progressbar.pulse()
-            while gtk.events_pending():
-                gtk.main_iteration()
+    def end(self, combo, progressbar, callback, dbname):
+        if not self.updated.isSet():
+            progressbar.show()
+            progressbar.pulse()
+            return True
         progressbar.hide()
         dbs, createdb = self.db_info
 
         if dbs is None:
-            return None, False
+            dbs, createdb = None, False
         elif dbs == -1:
-            return -1, False
+            dbs, createdb = -1, False
+        else:
+            from tryton.gui.main import Main
+            Main.get_main().refresh_ssl()
+            liststore = combo.get_model()
+            liststore.clear()
+            index = -1
+            for db_num, db_name in enumerate(dbs):
+                liststore.append([db_name])
+                if db_name == dbname:
+                    index = db_num
+            if index == -1:
+                index = 0
+            combo.set_active(index)
+            dbs = len(dbs)
 
-        from tryton.gui.main import Main
-        Main.get_main().refresh_ssl()
-        liststore = combo.get_model()
-        liststore.clear()
-        index = -1
-        for db_num, db_name in enumerate(dbs):
-            liststore.append([db_name])
-            if db_name == dbname:
-                index = db_num
-        if index == -1:
-            index = 0
-        combo.set_active(index)
-
-        return len(dbs), createdb
+        callback(dbs, createdb)
+        return False
 
 
 class RPCProgress(object):
 
-    def __init__(self, method, args, parent):
+    def __init__(self, method, args):
         self.method = method
         self.args = args
-        self.parent = parent
+        self.parent = get_toplevel_window()
         self.res = None
         self.error = False
         self.exception = None
@@ -1157,6 +1197,8 @@ class RPCProgress(object):
 
         watch = gtk.gdk.Cursor(gtk.gdk.WATCH)
         self.parent.window.set_cursor(watch)
+        parent_sensitive = self.parent.props.sensitive
+        self.parent.props.sensitive = False
         i = 0
         win = None
         progressbar = None
@@ -1200,10 +1242,12 @@ class RPCProgress(object):
                     win.window.set_cursor(watch)
                 with gtk.gdk.lock:
                     progressbar.pulse()
-            with gtk.gdk.lock:
-                while gtk.events_pending():
-                    gtk.main_iteration()
+            if win:
+                with gtk.gdk.lock:
+                    while gtk.events_pending():
+                        gtk.main_iteration()
             time.sleep(0.1)
+        self.parent.props.sensitive = parent_sensitive
         self.parent.window.set_cursor(None)
         if win:
             win.destroy()
@@ -1272,7 +1316,7 @@ def text_to_float_time(text, conv=None):
     try:
         try:
             return locale.atof(text)
-        except Exception:
+        except ValueError:
             pass
         if conv:
             tmp_conv = FLOAT_TIME_CONV.copy()
@@ -1298,7 +1342,7 @@ def text_to_float_time(text, conv=None):
             try:
                 value += abs(locale.atof(buf))
                 continue
-            except Exception:
+            except ValueError:
                 pass
             for sep in conv.keys():
                 if buf.endswith(sep):
@@ -1307,7 +1351,7 @@ def text_to_float_time(text, conv=None):
         if text.startswith('-'):
             value *= -1
         return value
-    except Exception:
+    except ValueError:
         return 0.0
 
 def float_time_to_text(val, conv=None):
@@ -1375,6 +1419,7 @@ _ALLOWED_CODES = set(dis.opmap[x] for x in [
     'STORE_MAP', 'LOAD_NAME', 'CALL_FUNCTION', 'COMPARE_OP', 'LOAD_ATTR',
     'STORE_NAME', 'GET_ITER', 'FOR_ITER', 'LIST_APPEND', 'JUMP_ABSOLUTE',
     'DELETE_NAME', 'JUMP_IF_TRUE', 'JUMP_IF_FALSE', 'JUMP_IF_FALSE_OR_POP',
+    'JUMP_IF_TRUE_OR_POP', 'POP_JUMP_IF_FALSE', 'POP_JUMP_IF_TRUE',
     'BINARY_SUBSCR',
     ] if x in dis.opmap)
 
@@ -1415,3 +1460,12 @@ def safe_eval(source, data=None):
         'bool': bool,
         'dict': dict,
         }}, data)
+
+def timezoned_date(date):
+    if pytz and rpc.CONTEXT.get('timezone'):
+        lzone = pytz.timezone(rpc.CONTEXT['timezone'])
+        szone = pytz.timezone(rpc.TIMEZONE)
+        sdt = szone.localize(date, is_dst=True)
+        ldt = sdt.astimezone(lzone)
+        date = ldt
+    return date
