@@ -6,21 +6,21 @@ from record import Record
 from field import Field, O2MField
 from tryton.signal_event import SignalEvent
 import tryton.common as common
+from tryton.exceptions import TrytonServerError
 from tryton.common.domain_inversion import is_leaf
 
 
 class Group(SignalEvent, list):
 
-    def __init__(self, model_name, fields, window, ids=None, parent=None,
+    def __init__(self, model_name, fields, ids=None, parent=None,
             parent_name='', child_name='', context=None, domain=None,
             readonly=False, parent_datetime_field=None):
         super(Group, self).__init__()
         if domain is None:
             domain = []
-        self.domain = domain
+        self.__domain = domain
         self.__domain4inversion = None
         self.lock_signal = False
-        self.__window = window
         self.parent = parent
         self.parent_name = parent_name or ''
         self.child_name = child_name
@@ -40,15 +40,12 @@ class Group(SignalEvent, list):
         self.__field_childs = None
         self.exclude_field = None
 
-    def __get_window(self):
-        return self.__window
-
-    def __set_window(self, window):
-        for record in self:
-            record.window = window
-        self.__window = window
-
-    window = property(__get_window, __set_window)
+    @property
+    def domain(self):
+        if self.parent and self.child_name:
+            field = self.parent.group.fields[self.child_name]
+            return [self.__domain, field.domain_get(self.parent)]
+        return self.__domain
 
     def clean4inversion(self, domain):
         "This method will replace non relevant fields for domain inversion"
@@ -155,6 +152,9 @@ class Group(SignalEvent, list):
             saved.append(record.save(force_reload=False))
         return saved
 
+    def delete(self, records, context=None):
+        return Record.delete(records, context=context)
+
     @property
     def root_group(self):
         root = self
@@ -179,16 +179,16 @@ class Group(SignalEvent, list):
 
     def on_write_ids(self, ids):
         if not self.on_write:
-            return False
+            return []
         res = []
         for fnct in self.on_write:
             args = ('model', self.model_name, fnct, ids, self.context)
             try:
                 res += rpc.execute(*args)
-            except Exception, exception:
-                res2 = common.process_exception(exception, self.window, *args)
+            except TrytonServerError, exception:
+                res2 = common.process_exception(exception, *args)
                 if not res2:
-                    return False
+                    return []
                 res += res2
         return list({}.fromkeys(res))
 
@@ -203,8 +203,7 @@ class Group(SignalEvent, list):
         for id in ids:
             if self.get(id):
                 continue
-            new_record = Record(self.model_name, id, self.window,
-                group=self)
+            new_record = Record(self.model_name, id, group=self)
             self.append(new_record)
             new_records.append(new_record)
             new_record.signal_connect(self, 'record-changed',
@@ -247,7 +246,6 @@ class Group(SignalEvent, list):
         if record.group is not self:
             record.signal_unconnect(record.group)
             record.group = self
-            record.window = self.window
             record.signal_connect(self, 'record-changed', self._record_changed)
             record.signal_connect(self, 'record-modified', self._record_modified)
         if position == -1:
@@ -279,7 +277,7 @@ class Group(SignalEvent, list):
 
     def new(self, default=True, domain=None, context=None, signal=True,
             obj_id=None):
-        record = Record(self.model_name, obj_id, self.window, group=self)
+        record = Record(self.model_name, obj_id, group=self)
         record.signal_connect(self, 'record-changed', self._record_changed)
         record.signal_connect(self, 'record-modified', self._record_modified)
         if default:
@@ -302,7 +300,7 @@ class Group(SignalEvent, list):
     def remove(self, record, remove=False, modified=True, signal=True,
             force_remove=False):
         idx = self.index(record)
-        if self[idx].id > 0:
+        if self[idx].id >= 0:
             if remove:
                 if self[idx] in self.record_deleted:
                     self.record_deleted.remove(self[idx])
@@ -317,7 +315,7 @@ class Group(SignalEvent, list):
         if modified:
             record.modified_fields.setdefault('id')
             record.signal('record-modified')
-        if not record.parent or self[idx].id <= 0 or force_remove:
+        if not record.parent or self[idx].id < 0 or force_remove:
             self._remove(self[idx])
 
         if len(self):
@@ -369,7 +367,7 @@ class Group(SignalEvent, list):
 
         new = []
         for record in self:
-            if record.id <= 0:
+            if record.id < 0:
                 new.append(record)
         ctx = context.copy()
 
@@ -378,8 +376,8 @@ class Group(SignalEvent, list):
             args = ('model', self.model_name, 'default_get', to_add.keys(), ctx)
             try:
                 values = rpc.execute(*args)
-            except Exception, exception:
-                values = common.process_exception(exception, self.window, *args)
+            except TrytonServerError, exception:
+                values = common.process_exception(exception, *args)
                 if not values:
                     return False
             for name in to_add:
@@ -400,7 +398,6 @@ class Group(SignalEvent, list):
 
     def destroy(self):
         super(Group, self).destroy()
-        self.__window = None
         self.parent = None
         self.fields = {}
         self.record_deleted, self.record_removed = [], []
