@@ -8,17 +8,21 @@ import field
 import datetime
 import logging
 import time
+from functools import reduce
+from tryton.exceptions import TrytonServerError
 
 
 class Record(SignalEvent):
 
     id = -1
 
-    def __init__(self, model_name, obj_id, window, group=None):
+    def __init__(self, model_name, obj_id, group=None):
         super(Record, self).__init__()
-        self.__window = window
         self.model_name = model_name
-        self.id = obj_id or Record.id
+        if obj_id is None:
+            self.id = Record.id
+        else:
+            self.id = obj_id
         if self.id < 0:
             Record.id -= 1
         self._loaded = set()
@@ -34,7 +38,7 @@ class Record(SignalEvent):
         self.autocompletion = {}
 
     def __getitem__(self, name, raise_exception=False):
-        if name not in self._loaded and self.id > 0:
+        if name not in self._loaded and self.id >= 0:
             ids =  [self.id]
             if name == '*':
                 loading = reduce(
@@ -52,11 +56,11 @@ class Record(SignalEvent):
                         idx + n < length) and n < 100:
                     if idx - n >= 0:
                         record = self.group[idx - n]
-                        if name not in record._loaded and record.id > 0:
+                        if name not in record._loaded and record.id >= 0:
                             ids.append(record.id)
                     if idx + n < length:
                         record = self.group[idx + n]
-                        if name not in record._loaded and record.id > 0:
+                        if name not in record._loaded and record.id >= 0:
                             ids.append(record.id)
                     n += 1
             if loading == 'eager':
@@ -71,13 +75,16 @@ class Record(SignalEvent):
             fields.append('_timestamp')
             ctx = rpc.CONTEXT.copy()
             ctx.update(self.context_get())
+            ctx.update(dict(('%s.%s' % (self.model_name, fname), 'size')
+                    for fname, field in self.group.fields.iteritems()
+                    if field.attrs['type'] == 'binary'))
             args = ('model', self.model_name, 'read', ids, fields, ctx)
             try:
                 values = rpc.execute(*args)
-            except Exception, exception:
+            except TrytonServerError, exception:
                 if raise_exception:
                     raise
-                values = common.process_exception(exception, self.window, *args)
+                values = common.process_exception(exception, *args)
                 if not values:
                     values = [{'id': x} for x in ids]
             id2value = dict((value['id'], value) for value in values)
@@ -127,8 +134,6 @@ class Record(SignalEvent):
         group.on_write = self.group.on_write
         group.readonly = self.group.readonly
         group._context.update(self.group._context)
-        if group.window != self.window:
-            group.window = self.window
         return group
 
     def get_path(self, group):
@@ -144,19 +149,6 @@ class Record(SignalEvent):
         path.reverse()
         return tuple(path)
 
-    def _get_window(self):
-        return self.__window
-
-    def _set_window(self, window):
-        self.__window = window
-        for fieldname, value in self.value.iteritems():
-            if fieldname not in self.group.fields:
-                continue
-            if isinstance(self.group.fields[fieldname], field.O2MField):
-                value.window = window
-
-    window = property(_get_window, _set_window)
-
     def get_removed(self):
         if self.group is not None:
             return self in self.group.record_removed
@@ -245,9 +237,8 @@ class Record(SignalEvent):
                     self.context_get())
                 try:
                     res = rpc.execute(*args)
-                except Exception, exception:
-                    res = common.process_exception(exception, self.window,
-                            *args)
+                except TrytonServerError, exception:
+                    res = common.process_exception(exception, *args)
                     if not res:
                         return False
                 old_id = self.id
@@ -266,9 +257,8 @@ class Record(SignalEvent):
                     try:
                         if not rpc.execute(*args):
                             return False
-                    except Exception, exception:
-                        res = common.process_exception(exception, self.window,
-                                *args)
+                    except TrytonServerError, exception:
+                        res = common.process_exception(exception, *args)
                         if not res:
                             return False
             self._loaded.clear()
@@ -282,14 +272,43 @@ class Record(SignalEvent):
             self.parent.save(force_reload=force_reload)
         return self.id
 
+    @staticmethod
+    def delete(records, context=None):
+        if not records:
+            return
+        record = records[0]
+        group = record.group
+        assert all(r.model_name == record.model_name for r in records)
+        assert all(r.group == group for r in records)
+        records = [r for r in records if r.id >= 0]
+        ctx = {}
+        ctx.update(rpc.CONTEXT)
+        ctx.update(context or {})
+        ctx['_timestamp'] = {}
+        for rec in records:
+            ctx['_timestamp'].update(rec.get_timestamp())
+        record_ids = set(r.id for r in records)
+        reload_ids = set(group.on_write_ids(list(record_ids)))
+        reload_ids -= record_ids
+        reload_ids = list(reload_ids)
+        args = ('model', record.model_name, 'delete', list(record_ids), ctx)
+        try:
+            rpc.execute(*args)
+        except TrytonServerError, exception:
+            if not common.process_exception(exception, *args):
+                return False
+        if reload_ids:
+            group.root_group.reload(reload_ids)
+        return True
+
     def default_get(self, domain=None, context=None):
         if len(self.group.fields):
             args = ('model', self.model_name, 'default_get',
                     self.group.fields.keys(), context)
             try:
                 vals = rpc.execute(*args)
-            except Exception, exception:
-                vals = common.process_exception(exception, self.window, *args)
+            except TrytonServerError, exception:
+                vals = common.process_exception(exception, *args)
                 if not vals:
                     return
             if (self.parent
@@ -309,8 +328,8 @@ class Record(SignalEvent):
         args = ('model', self.model_name, 'read', self.id, ['rec_name'], ctx)
         try:
             res = rpc.execute(*args)
-        except Exception, exception:
-            res = common.process_exception(exception, self.window, *args)
+        except TrytonServerError, exception:
+            res = common.process_exception(exception, *args)
             if not res:
                 return ''
         return res['rec_name']
@@ -436,12 +455,7 @@ class Record(SignalEvent):
 
     def _get_on_change_args(self, args):
         res = {}
-        values = {}
-        for name, field in self.group.fields.iteritems():
-            values[name] = field.get_on_change_value(self, check_load=False)
-        if self.parent and self.parent_name:
-            values['_parent_' + self.parent_name] = \
-                    common.EvalEnvironment(self.parent, False, 'on_change')
+        values = common.EvalEnvironment(self, True, 'on_change')
         for arg in args:
             scope = values
             for i in arg.split('.'):
@@ -461,8 +475,8 @@ class Record(SignalEvent):
         args = ('model', self.model_name, 'on_change_' + fieldname, args, ctx)
         try:
             res = rpc.execute(*args)
-        except Exception, exception:
-            res = common.process_exception(exception, self.window, *args)
+        except TrytonServerError, exception:
+            res = common.process_exception(exception, *args)
             if not res:
                 return
         later = {}
@@ -511,8 +525,8 @@ class Record(SignalEvent):
                     args, ctx)
             try:
                 res = rpc.execute(*args)
-            except Exception, exception:
-                res = common.process_exception(exception, self.window, *args)
+            except TrytonServerError, exception:
+                res = common.process_exception(exception, *args)
                 if not res:
                     return
             self.group.fields[fieldname].set_on_change(self, res)
@@ -534,8 +548,8 @@ class Record(SignalEvent):
             ctx)
         try:
             res = rpc.execute(*args)
-        except Exception, exception:
-            res = common.process_exception(exception, self.window, *args)
+        except TrytonServerError, exception:
+            res = common.process_exception(exception, *args)
             if not res:
                 # ensure res is a list
                 res = []
@@ -548,8 +562,8 @@ class Record(SignalEvent):
                 field_name + '=' + str(value), ctx)
         try:
             res = rpc.execute(*args)
-        except Exception, exception:
-            res = common.process_exception(exception, self.window, *args)
+        except TrytonServerError, exception:
+            res = common.process_exception(exception, *args)
             if not res:
                 return
         self.set_default(res)
@@ -563,13 +577,12 @@ class Record(SignalEvent):
                 ], rpc.CONTEXT)
             try:
                 self.attachment_count = rpc.execute(*args)
-            except Exception:
+            except TrytonServerError:
                 return 0
         return self.attachment_count
 
     def destroy(self):
         super(Record, self).destroy()
-        self.window = None
         self.group = None
         self.value = None
         self.next = None
