@@ -2,25 +2,33 @@
 #this repository contains the full copyright notices and license terms.
 "Screen"
 import gobject
+try:
+    import simplejson as json
+except ImportError:
+    import json
 import copy
+import collections
 import xml.dom.minidom
 import tryton.rpc as rpc
 from tryton.gui.window.view_form.model.group import Group
+from tryton.gui.window.view_form.model.record import Record
 from tryton.gui.window.view_form.view.screen_container import ScreenContainer
-from tryton.gui.window.view_form.widget_search import Form
 from tryton.signal_event import SignalEvent
 from tryton.common import node_attributes
 from tryton.config import CONFIG
 import tryton.common as common
+from tryton.exceptions import TrytonServerError, TrytonServerUnavailable
+from tryton.jsonrpc import JSONEncoder
+from tryton.common.tdp import DomainParser
 
 
 class Screen(SignalEvent):
     "Screen"
 
-    def __init__(self, model_name, window, view_ids=None, mode=None,
-            context=None, views_preload=None, domain=None, row_activate=None,
-            limit=None, readonly=False, exclude_field=None, sort=None,
-            search_value=None, alternate_view=False):
+    def __init__(self, model_name, view_ids=None, mode=None, context=None,
+            views_preload=None, domain=None, row_activate=None, limit=None,
+            readonly=False, exclude_field=None, sort=None, search_value=None,
+            alternate_view=False):
         if view_ids is None:
             view_ids = []
         if mode is None:
@@ -32,6 +40,8 @@ class Screen(SignalEvent):
         if domain is None:
             domain = []
 
+        self.limit = limit or int(CONFIG['client.limit'])
+        self.offset = 0
         super(Screen, self).__init__()
 
         self.readonly = readonly
@@ -45,31 +55,32 @@ class Screen(SignalEvent):
         self.model_name = model_name
         self.context = context
         self.views = []
-        self.view_ids = view_ids
+        self.view_ids = view_ids[:]
         self.parent = None
         self.parent_name = None
         self.exclude_field = exclude_field
-        self.__window = window
+        self.filter_widget = None
         self.__group = None
         self.new_group()
         self.__current_record = None
         self.current_record = None
         self.screen_container = ScreenContainer()
         self.screen_container.alternate_view = alternate_view
-        self.filter_widget = None
         self.widget = self.screen_container.widget_get()
         self.__current_view = 0
-        self.limit = limit
         self.search_value = search_value
         self.fields_view_tree = None
         self.sort = sort
         self.view_to_load = []
+        self.expanded_nodes = collections.defaultdict(
+            lambda: collections.defaultdict(lambda: None))
+        self.domain_parser = None
 
         if mode:
             self.view_to_load = mode[1:]
             view_id = False
-            if view_ids:
-                view_id = view_ids.pop(0)
+            if self.view_ids:
+                view_id = self.view_ids.pop(0)
             view = self.add_view_id(view_id, mode[0])
             self.screen_container.set(view.widget)
         self.display()
@@ -79,83 +90,97 @@ class Screen(SignalEvent):
 
     def search_active(self, active=True):
         if active and not self.parent:
-            if not self.filter_widget:
-                if not self.fields_view_tree:
-                    ctx = {}
-                    ctx.update(rpc.CONTEXT)
-                    ctx.update(self.context)
-                    try:
-                        self.fields_view_tree = rpc.execute('model',
-                                self.model_name, 'fields_view_get', False,
-                                'tree', ctx)
-                    except Exception:
-                        return
-                self.filter_widget = Form(self.fields_view_tree,
-                        self.model_name, self.window, self.domain,
-                        (self, self.search_filter), self.context)
-                self.screen_container.add_filter(self.filter_widget.widget,
-                        self.search_filter, self.search_clear,
-                        self.search_prev, self.search_next)
-                self.filter_widget.set_limit(self.limit)
-                self.filter_widget.value = self.search_value
+            if not self.fields_view_tree:
+                ctx = {}
+                ctx.update(rpc.CONTEXT)
+                ctx.update(self.context)
+                try:
+                    self.fields_view_tree = rpc.execute('model',
+                            self.model_name, 'fields_view_get', False,
+                            'tree', ctx)
+                except TrytonServerError:
+                    return
+
+            fields = self.fields_view_tree['fields']
+            for name, props in fields.iteritems():
+                if props['type'] not in ('selection', 'reference'):
+                    continue
+                if isinstance(props['selection'], (tuple, list)):
+                    continue
+                props['selection'] = self.get_selection(props)
+
+            self.domain_parser = DomainParser(
+                self.fields_view_tree['fields'])
+
+            self.screen_container.set_screen(self)
             self.screen_container.show_filter()
         else:
             self.screen_container.hide_filter()
 
-    def search_prev(self, widget=None):
-        self.filter_widget.prev()
-        self.search_filter()
+    def get_selection(self, props):
+        try:
+            selection = rpc.execute('model',
+                    self.model_name, props['selection'], rpc.CONTEXT)
+        except TrytonServerError, exception:
+            common.process_exception(exception, None)
+            selection = []
+        selection.sort(lambda x, y: cmp(x[1], y[1]))
+        return selection
 
-    def search_next(self, widget=None):
-        self.filter_widget.next()
-        self.search_filter()
+    def search_prev(self, search_string):
+        self.offset -= self.limit
+        self.search_filter(search_string=search_string)
 
-    def search_clear(self, widget=None):
-        self.filter_widget.clear()
-        self.clear()
+    def search_next(self, search_string):
+        self.offset += self.limit
+        self.search_filter(search_string=search_string)
 
-    def search_filter(self, widget=None, only_ids=False):
-        limit = None
-        offset = 0
-        values = []
-        if self.filter_widget:
-            limit = self.filter_widget.get_limit()
-            offset = self.filter_widget.get_offset()
-            values = self.filter_widget.value
+    def search_complete(self, search_string):
+        parsed_tree = self.domain_parser.parse(search_string)
+        res = list(r for r in parsed_tree.complete() \
+                if r.strip() != search_string)
+        return res
+
+    def search_filter(self, search_string=None, only_ids=False):
+        domain = []
+
+        if self.domain_parser and not self.parent:
+            if search_string is not None:
+                domain = self.domain_parser.parse(search_string or '').domain()
+            else:
+                domain = self.search_value
+            self.screen_container.set_text(self.domain_parser.string(domain))
         else:
-            values = [('id', 'in', [x.id for x in self.group])]
+            domain = [('id', 'in', [x.id for x in self.group])]
+
         ctx = {}
         ctx.update(rpc.CONTEXT)
         ctx.update(self.context)
-        if values:
+        if domain:
             if self.domain:
-                values = ['AND', values, self.domain]
+                domain = ['AND', domain, self.domain]
         else:
-            values = self.domain
+            domain = self.domain
+        rpc_args = ('model', self.model_name, 'search', domain, self.offset,
+            self.limit, self.sort, ctx)
         try:
-            try:
-                ids = rpc.execute('model', self.model_name, 'search', values,
-                        offset, limit, self.sort, ctx)
-            except Exception, exception:
-                common.process_exception(exception, self.window)
-                ids = rpc.execute('model', self.model_name, 'search', values,
-                        offset, limit, self.sort, ctx)
-            if not only_ids:
-                if len(ids) == limit:
-                    try:
-                        self.search_count = rpc.execute('model',
-                                self.model_name, 'search_count', values, ctx)
-                    except Exception, exception:
-                        common.process_exception(exception, self.window)
-                        self.search_count = rpc.execute('model',
-                                self.model_name, 'search_count', values, ctx)
-                else:
-                    self.search_count = len(ids)
-        except Exception:
-            ids = []
-        self.screen_container.but_prev.set_sensitive(bool(offset))
-        if (len(ids) == limit
-                and self.search_count > limit + offset):
+            ids = rpc.execute(*rpc_args)
+        except TrytonServerError, exception:
+            ids = (common.process_exception(exception, *rpc_args) or [])
+        if not only_ids:
+            if len(ids) == self.limit:
+                rpc_args = ('model', self.model_name, 'search_count', domain,
+                    ctx)
+                try:
+                    self.search_count = rpc.execute(*rpc_args)
+                except TrytonServerError, exception:
+                    self.search_count = (common.process_exception(exception,
+                            *rpc_args) or 0)
+            else:
+                self.search_count = len(ids)
+        self.screen_container.but_prev.set_sensitive(bool(self.offset))
+        if (len(ids) == self.limit
+                and self.search_count > self.limit + self.offset):
             self.screen_container.but_next.set_sensitive(True)
         else:
             self.screen_container.but_next.set_sensitive(False)
@@ -195,7 +220,7 @@ class Screen(SignalEvent):
     group = property(__get_group, __set_group)
 
     def new_group(self):
-        self.group = Group(self.model_name, {}, self.window, domain=self.domain,
+        self.group = Group(self.model_name, {}, domain=self.domain,
             context=self.context, readonly=self.readonly)
 
     def _group_cleared(self, group, signal):
@@ -220,11 +245,7 @@ class Screen(SignalEvent):
     def __set_current_record(self, record):
         self.__current_record = record
         try:
-            offset = int(self.filter_widget.get_offset())
-        except Exception:
-            offset = 0
-        try:
-            pos = self.group.index(record) + offset + 1
+            pos = self.group.index(record) + self.offset + 1
         except ValueError:
             pos = []
             i = record
@@ -233,7 +254,7 @@ class Screen(SignalEvent):
                 i = i.parent
             pos.reverse()
             pos = tuple(pos)
-        self.signal('record-message', (pos or 0, len(self.group) + offset,
+        self.signal('record-message', (pos or 0, len(self.group) + self.offset,
             self.search_count, record and record.id))
         attachment_count = 0
         if record and record.attachment_count > 0:
@@ -245,15 +266,6 @@ class Screen(SignalEvent):
 
     current_record = property(__get_current_record, __set_current_record)
 
-    def __get_window(self):
-        return self.__window
-
-    def __set_window(self, window):
-        self.group.window = window
-        self.__window = window
-
-    window = property(__get_window, __set_window)
-
     def update_attachment(self, record):
         if record != self.current_record:
             return False
@@ -263,12 +275,12 @@ class Screen(SignalEvent):
         return False
 
     def destroy(self):
+        self.save_tree_state()
         for view in self.views:
             view.destroy()
         self.group.signal_unconnect(self)
         self.group.destroy()
         self.parent = None
-        self.__window = None
         self.__group = None
         self.__current_record = None
         self.screen_container = None
@@ -276,9 +288,10 @@ class Screen(SignalEvent):
 
     def default_row_activate(self):
         from tryton.action import Action
+
         if (self.current_view.view_type == 'tree' and
                 self.current_view.widget_tree.keyword_open):
-            return Action.exec_keyword('tree_open', self.window, {
+            return Action.exec_keyword('tree_open', {
                 'model': self.model_name,
                 'id': self.id_get(),
                 'ids': [self.id_get()],
@@ -300,17 +313,18 @@ class Screen(SignalEvent):
             self.current_view.set_cursor()
             self.current_view.display()
             return
-        for i in xrange(len(self.views) + len(self.view_to_load)):
-            if len(self.view_to_load):
-                self.load_view_to_load()
-                self.__current_view = len(self.views) - 1
-            else:
-                self.__current_view = ((self.__current_view + 1)
-                        % len(self.views))
-            if not view_type:
-                break
-            elif self.current_view.view_type == view_type:
-                break
+        if not view_type or self.current_view.view_type != view_type:
+            for i in xrange(len(self.views) + len(self.view_to_load)):
+                if len(self.view_to_load):
+                    self.load_view_to_load()
+                    self.__current_view = len(self.views) - 1
+                else:
+                    self.__current_view = ((self.__current_view + 1)
+                            % len(self.views))
+                if not view_type:
+                    break
+                elif self.current_view.view_type == view_type:
+                    break
         self.screen_container.set(self.current_view.widget)
         if not self.current_record and self.current_view.view_type == 'form':
             self.new(default=default, context=context)
@@ -335,21 +349,16 @@ class Screen(SignalEvent):
             ctx.update(rpc.CONTEXT)
             ctx.update(self.context)
             args = ('model', self.model_name, 'fields_view_get',
-                    view_id, view_type,
-                    self.parent and False or CONFIG['form.toolbar'],
-                    ctx)
+                    view_id, view_type, ctx)
             try:
                 view = rpc.execute(*args)
-            except Exception, exception:
-                view = common.process_exception(exception, self.window, *args)
+            except TrytonServerError, exception:
+                view = common.process_exception(exception, *args)
                 if not view:
                     return
-        return self.add_view(view, display, toolbar=view.get('toolbar', False),
-                context=context)
+        return self.add_view(view, display, context=context)
 
-    def add_view(self, view, display=False, toolbar=None, context=None):
-        if toolbar is None:
-            toolbar = {}
+    def add_view(self, view, display=False, context=None):
         arch = view['arch']
         fields = view['fields']
 
@@ -372,8 +381,8 @@ class Screen(SignalEvent):
         from tryton.gui.window.view_form.view.widget_parse import WidgetParse
         self.group.add_fields(fields, context=context)
 
-        parser = WidgetParse(parent=self.parent, window=self.window)
-        view = parser.parse(self, xml_dom, self.group.fields, toolbar=toolbar,
+        parser = WidgetParse(parent=self.parent)
+        view = parser.parse(self, xml_dom, self.group.fields,
                 children_field=children_field)
 
         self.views.append(view)
@@ -475,6 +484,7 @@ class Screen(SignalEvent):
             self.current_view.set_cursor()
             self.current_view.display()
             return False
+        self.signal('record-saved')
         if path and obj_id:
             path = path[:-1] + ((path[-1][0], obj_id),)
         self.current_record = self.group.get_by_path(path)
@@ -499,13 +509,13 @@ class Screen(SignalEvent):
                 get_modifiedonly=get_modifiedonly)
 
     def modified(self):
-        self.current_view.set_value()
         res = False
         if self.current_view.view_type != 'tree':
-            res = self.current_record and self.current_record.modified
+            if self.current_record:
+                res = self.current_record.modified or self.current_record.id < 0
         else:
             for record in self.group:
-                if record.modified:
+                if record.modified or record.id < 0:
                     res = True
         return res
 
@@ -525,110 +535,101 @@ class Screen(SignalEvent):
             self.group.unremove(record)
 
     def remove(self, delete=False, remove=False, force_remove=False):
-        res = False
-        reload_ids = []
+        records = None
         if self.current_view.view_type == 'form' and self.current_record:
-            record_id = self.current_record.id
-            if delete and record_id > 0:
-                context = {}
-                context.update(rpc.CONTEXT)
-                context.update(self.context)
-                context['_timestamp'] = self.current_record.get_timestamp()
-                reload_ids = self.group.on_write_ids([record_id])
-                if reload_ids and record_id in reload_ids:
-                    reload_ids.remove(record_id)
-                args = ('model', self.model_name, 'delete', [record_id],
-                        context)
-                try:
-                    res = rpc.execute(*args)
-                except Exception, exception:
-                    res = common.process_exception(exception, self.window,
-                            *args)
-                if not res:
-                    return False
-            self.current_view.set_cursor()
-            record = self.current_record
-            idx = record.group.index(record)
-            record.group.remove(record, remove=remove,
-                force_remove=force_remove)
+            records = [self.current_record]
+        elif self.current_view.view_type == 'tree':
+            records = self.current_view.selected_records()
+        if delete:
+            if not self.group.delete(records):
+                return False
 
-            if delete:
-                if (record.parent and
-                        record.parent.model_name == record.model_name):
+        top_record = records[0]
+        idx = top_record.group.index(top_record)
+        path = top_record.get_path(self.group)
+
+        for record in records:
+            # set current model to None to prevent __select_changed
+            # to save the previous_model as it can be already deleted.
+            self.current_record = None
+            record.group.remove(record, remove=remove, signal=False,
+                force_remove=force_remove)
+        # send record-changed only once
+        record.signal('record-changed')
+
+        if delete:
+            for record in records:
+                if record.parent:
                     record.parent.save()
 
-            if record.group:
-                idx = min(idx, len(record.group) - 1)
-                self.current_record = record.group[idx]
-            elif (record.parent and
-                    record.parent.model_name == record.model_name):
-                self.current_record = record.parent
-            else:
-                self.current_record = None
-            if reload_ids:
-                self.group.root_group.reload(reload_ids)
-            self.display()
-            res = True
-        if self.current_view.view_type == 'tree':
-            records = self.current_view.selected_records()
-            saved_records = [r for r in records if r.id >= 0]
-            if delete and saved_records:
-                context = {}
-                context.update(rpc.CONTEXT)
-                context.update(self.context)
-                context['_timestamp'] = {}
-                for record in saved_records:
-                    context['_timestamp'].update(record.get_timestamp())
-                reload_ids = self.group.on_write_ids([x.id for x in saved_records])
-                if reload_ids:
-                    for record in saved_records:
-                        if record.id in reload_ids:
-                            reload_ids.remove(record.id)
-                args = ('model', self.model_name, 'delete',
-                        [x.id for x in saved_records], context)
-                try:
-                    res = rpc.execute(*args)
-                except Exception, exception:
-                    res = common.process_exception(exception, self.window,
-                            *args)
-                if not res:
-                    return False
-            if not records:
-                return True
-            path = self.current_view.store.on_get_path(records[0])
-            for record in records:
-                # set current model to None to prevent __select_changed
-                # to save the previous_model as it can be already deleted.
-                self.current_record = None
-                record.group.remove(record, remove=remove, signal=False,
-                    force_remove=force_remove)
-
-            # send record-changed only once
-            record.signal('record-changed')
-
-            if delete:
-                for record in records:
-                    if record.parent:
-                        record.parent.save()
-
-            if path[-1] > 0:
-                path = path[:-1] + (path[-1] - 1,)
-            else:
-                path = path[:-1]
-            if path:
-                iter_ = self.current_view.store.get_iter(path)
-                self.current_record = self.current_view.store.get_value(iter_, 0)
-            elif len(self.group):
-                self.current_record = self.group[0]
-            if reload_ids:
-                self.group.root_group.reload(reload_ids)
-            self.current_view.set_cursor()
-            self.display()
-            res = True
+        if idx > 0:
+            record = top_record.group[idx - 1]
+            path = path[:-1] + ((path[-1][0], record.id,),)
+        else:
+            path = path[:-1]
+        if path:
+            self.current_record = self.group.get_by_path(path)
+        elif len(self.group):
+            self.current_record = self.group[0]
+        self.current_view.set_cursor()
+        self.display()
         self.request_set()
-        return res
+        return True
+
+    def set_tree_state(self):
+        view = self.current_view
+        if (not CONFIG['client.save_tree_expanded_state']
+                or not self.current_view
+                or self.current_view.view_type != 'tree'
+                or not self.current_view.children_field
+                or not self.group):
+            return
+        parent = self.parent.id if self.parent else None
+        expanded_nodes = self.expanded_nodes[parent][view.children_field]
+        if expanded_nodes is None:
+            json_domain = self.get_tree_domain(parent)
+            try:
+                expanded_nodes = rpc.execute('model',
+                    'ir.ui.view_tree_expanded_state', 'get_expanded',
+                    self.model_name, json_domain,
+                    self.current_view.children_field, rpc.CONTEXT)
+                expanded_nodes = json.loads(expanded_nodes)
+            except TrytonServerError:
+                expanded_nodes = []
+            self.expanded_nodes[parent][view.children_field] = expanded_nodes
+        view.expand_nodes(expanded_nodes)
+
+    def save_tree_state(self):
+        view = self.current_view
+        if (not CONFIG['client.save_tree_expanded_state']
+                or not view
+                or view.view_type != 'tree'
+                or not view.children_field
+                or not (self.parent is None
+                    or isinstance(self.parent, Record))):
+            return
+        parent = self.parent.id if self.parent else None
+        paths = view.get_expanded_paths()
+        self.expanded_nodes[parent][view.children_field] = paths
+        json_domain = self.get_tree_domain(parent)
+        json_paths = json.dumps(paths)
+        try:
+            rpc.execute('model', 'ir.ui.view_tree_expanded_state',
+                'set_expanded', self.model_name, json_domain,
+                self.current_view.children_field, json_paths, rpc.CONTEXT)
+        except (TrytonServerError, TrytonServerUnavailable):
+            pass
+
+    def get_tree_domain(self, parent):
+        if parent:
+            domain = (self.domain + [(self.exclude_field, '=', parent)])
+        else:
+            domain = self.domain
+        json_domain = json.dumps(domain, cls=JSONEncoder)
+        return json_domain
 
     def load(self, ids, set_cursor=True, modified=False):
+        self.expanded_nodes.clear()
         self.group.load(ids, display=False, modified=modified)
         self.current_view.reset()
         if ids:
@@ -658,6 +659,7 @@ class Screen(SignalEvent):
                     in ('tree', 'graph', 'calendar'))
             if set_cursor:
                 self.current_view.set_cursor(reset_view=False)
+        self.set_tree_state()
 
     def display_next(self):
         view = self.current_view
