@@ -7,9 +7,13 @@ import locale
 class Integer(Char):
     "Integer"
 
-    def __init__(self, field_name, model_name, window, attrs=None):
-        super(Integer, self).__init__(field_name, model_name, window,
-                attrs=attrs)
+    def __init__(self, field_name, model_name, attrs=None):
+        super(Integer, self).__init__(field_name, model_name, attrs=attrs)
+        self.entry.set_width_chars(8)
+        _, _, padding, pack_type = self.widget.query_child_packing(
+            self.entry)
+        self.widget.set_child_packing(self.entry, False, False,
+            padding, pack_type)
         self.entry.set_max_length(0)
         self.entry.set_alignment(1.0)
         self.entry.connect('insert_text', self.sig_insert_text)
@@ -17,7 +21,7 @@ class Integer(Char):
     def set_value(self, record, field):
         try:
             value = locale.atoi(self.entry.get_text())
-        except:
+        except ValueError:
             value = 0
         return field.set_client(record, value)
 
@@ -30,16 +34,13 @@ class Integer(Char):
         self.entry.set_text(locale.format('%d',
             field.get(record) or 0, True))
 
-    def display_value(self):
-        return self.entry.get_text()
-
     def sig_insert_text(self, entry, new_text, new_text_length, position):
         value = entry.get_text()
         position = entry.get_position()
         new_value = value[:position] + new_text + value[position:]
+        if new_value == '-':
+            return
         try:
-            if new_value == '-':
-                return
             locale.atoi(new_value)
-        except:
+        except ValueError:
             entry.stop_emission('insert-text')
