@@ -1,12 +1,15 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+import operator
 import gtk
 import gobject
 import gettext
+import math
 from interface import WidgetInterface
 from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
 from tryton.gui.window.view_form.screen import Screen
+from tryton.exceptions import TrytonServerError
 import tryton.rpc as rpc
 import tryton.common as common
 from tryton.config import CONFIG
@@ -16,9 +19,8 @@ _ = gettext.gettext
 
 class Reference(WidgetInterface):
 
-    def __init__(self, field_name, model_name, window, attrs=None):
-        super(Reference, self).__init__(field_name, model_name, window,
-                attrs=attrs)
+    def __init__(self, field_name, model_name, attrs=None):
+        super(Reference, self).__init__(field_name, model_name, attrs=attrs)
 
         self.widget = gtk.HBox(spacing=0)
 
@@ -28,7 +30,6 @@ class Reference(WidgetInterface):
         child.connect('changed',
                 self.sig_changed_combo)
         child.connect('key_press_event', self.sig_key_pressed)
-        self.widget_combo.set_size_request(int(attrs.get('widget_size', -1)), -1)
         self.widget.pack_start(self.widget_combo, expand=False, fill=True)
 
         self.widget.pack_start(gtk.Label('-'), expand=False, fill=False)
@@ -80,10 +81,20 @@ class Reference(WidgetInterface):
             try:
                 selection = rpc.execute('model',
                         self.model_name, selection, rpc.CONTEXT)
-            except Exception, exception:
-                common.process_exception(exception, self.window)
+            except TrytonServerError, exception:
+                common.process_exception(exception)
                 selection = []
-        selection.sort(lambda x, y: cmp(x[1], y[1]))
+        selection.sort(key=operator.itemgetter(1))
+        if selection:
+            pop = sorted((len(x) for x in selection), reverse=True)
+            average = sum(pop) / len(pop)
+            deviation = int(math.sqrt(sum((x - average)**2 for x in pop) /
+                    len(pop)))
+            width = max(next((x for x in pop if x < (deviation * 4)), 10), 10)
+        else:
+            width = 10
+        child.set_width_chars(width)
+
         self.set_popdown(selection)
 
         self.last_key = (None, 0)
@@ -144,25 +155,28 @@ class Reference(WidgetInterface):
 
         self.focus_out = False
         if not value:
-            model, (obj_id, name) = self.get_model() or '', (0, '')
+            model, (obj_id, name) = self.get_model() or '', (-1, '')
         else:
             try:
                 model, (obj_id, name) = value
             except ValueError:
                 self.focus_out = True
-                return False
-        if model and obj_id:
+                return
+        if model and obj_id >= 0:
             if not leave:
-                screen = Screen(model, self.window, mode=['form'])
+                screen = Screen(model, mode=['form'])
                 screen.load([obj_id])
-                win = WinForm(screen, self.widget.get_toplevel())
-                if win.run():
-                    if screen.save_current():
+                def callback(result):
+                    if result and screen.save_current():
                         value = (screen.current_record.id,
                                 screen.current_record.rec_name())
                         self.field.set_client(self.record, (model, value),
                                 force_change=True)
-                win.destroy()
+                    self.focus_out = True
+                    self.changed = True
+                    self.display(self.record, self.field)
+                WinForm(screen, callback)
+                return
         elif model:
             if not self._readonly and ( self.wid_text.get_text() or not leave):
                 domain = self.field.domain_get(self.record)
@@ -177,25 +191,28 @@ class Reference(WidgetInterface):
                         dom = domain
                     ids = rpc.execute('model', model, 'search', dom, 0,
                             CONFIG['client.limit'], None, context)
-                except Exception, exception:
+                except TrytonServerError, exception:
                     self.focus_out = True
                     self.changed = True
-                    common.process_exception(exception, self.window)
-                    return False
+                    common.process_exception(exception)
+                    return
                 if ids and len(ids) == 1:
                     self.field.set_client(self.record, (model, (ids[0], '')))
                     self.display(self.record, self.field)
                     self.focus_out = True
                     self.changed = True
-                    return True
+                    return
 
-                win = WinSearch(model, sel_multi=False, ids=ids, context=context,
-                        domain=domain, parent=self.widget.get_toplevel())
-                ids = win.run()
-                if ids:
-                    self.field.set_client(self.record, (model, (ids[0], '')))
-        else:
-            self.field.set_client(self.record, ('', (name, name)))
+                def callback(ids):
+                    if ids:
+                        self.field.set_client(self.record, (model, (ids[0], '')))
+                    self.focus_out = True
+                    self.changed = True
+                    self.display(self.record, self.field)
+                WinSearch(model, callback, sel_multi=False, ids=ids,
+                        context=context, domain=domain)
+                return
+        self.field.set_client(self.record, ('', (name, name)))
         self.focus_out = True
         self.changed = True
         self.display(self.record, self.field)
@@ -204,14 +221,13 @@ class Reference(WidgetInterface):
         model = self.get_model()
         if not model:
             return
-        screen = Screen(model, self.window, mode=['form'])
-        win = WinForm(screen, self.widget.get_toplevel(), new=True)
-        if win.run():
-            if screen.save_current():
+        screen = Screen(model, mode=['form'])
+        def callback(result):
+            if result and screen.save_current():
                 value = (screen.current_record.id,
                         screen.current_record.rec_name())
                 self.field.set_client(self.record, (model, value))
-        win.destroy()
+        WinForm(screen, callback, new=True)
 
     def sig_key_press(self, widget, event):
         editable = self.wid_text.get_editable()
@@ -233,18 +249,19 @@ class Reference(WidgetInterface):
         if not self.changed:
             return
         self.wid_text.set_text('')
-        self.field.set_client(self.record, (self.get_model(), (0, '')))
+        self.wid_text.set_position(0)
+        self.field.set_client(self.record, (self.get_model(), (-1, '')))
 
     def sig_changed(self, *args):
         if not self.changed:
             return False
         val = self.field.get_client(self.record)
         if not val:
-            model, (obj_id, name) = '', (0, '')
+            model, (obj_id, name) = '', (-1, '')
         else:
             model, (obj_id, name) = val
-        if self.get_model() and obj_id:
-            self.field.set_client(self.record, (self.get_model(), (0, '')))
+        if self.get_model() and obj_id >= 0:
+            self.field.set_client(self.record, (self.get_model(), (-1, '')))
             self.display(self.record, self.field)
         return False
 
@@ -253,29 +270,22 @@ class Reference(WidgetInterface):
         self.changed = False
         if not field:
             child.set_text('')
+            child.set_position(0)
             self.changed =True
             return False
         super(Reference, self).display(record, field)
         value = field.get_client(record)
         img = gtk.Image()
         if not value:
-            model, (obj_id, name) = '', (0, '')
+            model, (obj_id, name) = '', (-1, '')
         else:
             model, (obj_id, name) = value
         if model:
             child.set_text(self._selection2[model])
-            if not name and obj_id:
-                args = ('model', model, 'read', obj_id, ['rec_name'],
-                        rpc.CONTEXT)
-                try:
-                    name = rpc.execute(*args)
-                except Exception, exception:
-                    name = common.process_exception(exception, self.window,
-                            *args)
-                    if not name:
-                        name = '???'
+            child.set_position(len(self._selection2[model]))
             self.wid_text.set_text(name)
-            if obj_id:
+            self.wid_text.set_position(len(name))
+            if obj_id >= 0:
                 img.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
                 self.but_open.set_image(img)
             else:
@@ -283,15 +293,13 @@ class Reference(WidgetInterface):
                 self.but_open.set_image(img)
         else:
             child.set_text('')
+            child.set_position(0)
             self.wid_text.set_text(str(name))
+            self.wid_text.set_position(len(str(name)))
             img.set_from_stock('tryton-find', gtk.ICON_SIZE_SMALL_TOOLBAR)
             self.but_open.set_image(img)
         self.changed = True
 
-    def display_value(self):
-        return self.widget_combo.get_child().get_text() + ', ' + \
-                self.wid_text.get_text()
-
     def sig_key_pressed(self, *args):
         key = args[1].string.lower()
         if self.last_key[0] == key:
