@@ -2,6 +2,7 @@
 #this repository contains the full copyright notices and license terms.
 #This code is inspired by the pycha project (http://www.lorenzogil.com/projects/pycha/)
 import gtk
+from functools import reduce
 from tryton.common import hex2rgb, generateColorscheme, DT_FORMAT, \
         DHM_FORMAT, COLOR_SCHEMES, datetime_strftime
 from tryton.pyson import PYSONDecoder
@@ -14,6 +15,7 @@ import tryton.rpc as rpc
 import cairo
 from tryton.action import Action
 from tryton.translate import date_format
+from tryton.gui.window import Window
 
 
 class Popup(object):
@@ -35,11 +37,11 @@ class Popup(object):
         widget_x, widget_y = widget.window.get_origin()
         width, height = widget.window.get_size()
         popup_width, popup_height = self.win.get_size()
-        if x < popup_width / 2:
-            x = popup_width / 2
-        if x > width - popup_width / 2:
-            x = width - popup_width / 2
-        pos_x = widget_x + x - popup_width / 2
+        if x < popup_width // 2:
+            x = popup_width // 2
+        if x > width - popup_width // 2:
+            x = width - popup_width // 2
+        pos_x = widget_x + x - popup_width // 2
         if pos_x < 0:
             pos_x = 0
         if y < popup_height + 5:
@@ -126,10 +128,10 @@ class Graph(gtk.DrawingArea):
 
         self.queue_draw()
 
-    def action(self, window):
+    def action(self):
         self.popup.hide()
 
-    def action_keyword(self, ids, window):
+    def action_keyword(self, ids):
         if not ids:
             return
         ctx = self.group.context.copy()
@@ -137,17 +139,23 @@ class Graph(gtk.DrawingArea):
             del ctx['active_ids']
         if 'active_id' in ctx:
             del ctx['active_id']
-        return Action.exec_keyword('graph_open', window, {
-            'model': self.model,
-            'id': ids[0],
-            'ids': ids,
-            }, context=ctx, warning=False)
+        event = gtk.get_current_event()
+        allow_similar = False
+        if (event.state & gtk.gdk.CONTROL_MASK
+                or event.state & gtk.gdk.MOD1_MASK):
+            allow_similar = True
+        with Window(hide_current=True, allow_similar=allow_similar):
+            return Action.exec_keyword('graph_open', {
+                    'model': self.model,
+                    'id': ids[0],
+                    'ids': ids,
+                    }, context=ctx, warning=False)
 
     def drawBackground(self, cr, width, height):
         # Fill the background
         cr.save()
         r, g, b = hex2rgb(self.attrs.get('background', '#d5d5d5'))
-        linear = cairo.LinearGradient(width / 2, 0, width / 2, height)
+        linear = cairo.LinearGradient(width // 2, 0, width // 2, height)
         linear.add_color_stop_rgb(0, 1, 1, 1)
         linear.add_color_stop_rgb(1, r, g, b)
         cr.set_source(linear)
@@ -352,11 +360,13 @@ class Graph(gtk.DrawingArea):
                 key = yfield.get('key', yfield['name'])
                 self.datas[x].setdefault(key, 0.0)
                 if yfield.get('domain'):
-                    values = rpc.CONTEXT.copy()
-                    values['state'] = 'draft'
+                    context = rpc.CONTEXT.copy()
+                    context['context'] = context.copy()
+                    context['_user'] = rpc._USER
                     for field in model.group.fields:
-                        values[field] = model[field].get(model, check_load=False)
-                    if not PYSONDecoder(values).decode(yfield['domain']):
+                        context[field] = model[field].get(model,
+                            check_load=False)
+                    if not PYSONDecoder(context).decode(yfield['domain']):
                         continue
                 if yfield['name'] == '#':
                     self.datas[x][key] += 1
